from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.external_payment_create_data_attributes_metadata import ExternalPaymentCreateDataAttributesMetadata
    from ..models.external_payment_create_data_attributes_options import ExternalPaymentCreateDataAttributesOptions


T = TypeVar("T", bound="ExternalPaymentCreateDataAttributes")


@attr.s(auto_attribs=True)
class ExternalPaymentCreateDataAttributes:
    """
    Attributes:
        payment_source_token (str): The payment source token, as generated by the external gateway SDK. Credit Card
            numbers are rejected. Example: xxxx.yyyy.zzzz.
        options (Union[Unset, ExternalPaymentCreateDataAttributesOptions]): External payment options. Example: {'foo':
            'bar'}.
        reference (Union[Unset, str]): A string that you can use to add any external identifier to the resource. This
            can be useful for integrating the resource to an external system, like an ERP, a marketing tool, a CRM, or
            whatever. Example: ANY-EXTERNAL-REFEFERNCE.
        reference_origin (Union[Unset, str]): Any identifier of the third party system that defines the reference code
            Example: ANY-EXTERNAL-REFEFERNCE-ORIGIN.
        metadata (Union[Unset, ExternalPaymentCreateDataAttributesMetadata]): Set of key-value pairs that you can attach
            to the resource. This can be useful for storing additional information about the resource in a structured
            format. Example: {'foo': 'bar'}.
    """

    payment_source_token: str
    options: Union[Unset, "ExternalPaymentCreateDataAttributesOptions"] = UNSET
    reference: Union[Unset, str] = UNSET
    reference_origin: Union[Unset, str] = UNSET
    metadata: Union[Unset, "ExternalPaymentCreateDataAttributesMetadata"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        payment_source_token = self.payment_source_token
        options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.options, Unset):
            options = self.options.to_dict()

        reference = self.reference
        reference_origin = self.reference_origin
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "payment_source_token": payment_source_token,
            }
        )
        if options is not UNSET:
            field_dict["options"] = options
        if reference is not UNSET:
            field_dict["reference"] = reference
        if reference_origin is not UNSET:
            field_dict["reference_origin"] = reference_origin
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.external_payment_create_data_attributes_metadata import (
            ExternalPaymentCreateDataAttributesMetadata,
        )
        from ..models.external_payment_create_data_attributes_options import ExternalPaymentCreateDataAttributesOptions

        d = src_dict.copy()
        payment_source_token = d.pop("payment_source_token")

        _options = d.pop("options", UNSET)
        options: Union[Unset, ExternalPaymentCreateDataAttributesOptions]
        if isinstance(_options, Unset):
            options = UNSET
        else:
            options = ExternalPaymentCreateDataAttributesOptions.from_dict(_options)

        reference = d.pop("reference", UNSET)

        reference_origin = d.pop("reference_origin", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, ExternalPaymentCreateDataAttributesMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = ExternalPaymentCreateDataAttributesMetadata.from_dict(_metadata)

        external_payment_create_data_attributes = cls(
            payment_source_token=payment_source_token,
            options=options,
            reference=reference,
            reference_origin=reference_origin,
            metadata=metadata,
        )

        external_payment_create_data_attributes.additional_properties = d
        return external_payment_create_data_attributes

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
